#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'hardware_information/class/constants.rb'
# =========================================================================== #
require 'hardware_information/project/project.rb'

class HardwareInformation

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === ERROR_LINE
  # ========================================================================= #
  ERROR_LINE = '2>&1'

  # ========================================================================= #
  # === LEFT_PADDING
  #
  # How many characters to pad on the left side. Used for the
  # header_content() method.
  # ========================================================================= #
  LEFT_PADDING = 26

  # ========================================================================= #
  # === MAIN_DEFAULT_RJUST
  # ========================================================================= #
  MAIN_DEFAULT_RJUST = 48

  # ========================================================================= #
  # === CPUINFO_LOCATION
  #
  # This constants stores where cpuinfo is kept. Will work only on
  # Linux, I think.
  # ========================================================================= #
  CPUINFO_LOCATION = '/proc/cpuinfo'

  # ========================================================================= #
  # === PROC_MEMINFO
  # ========================================================================= #
  PROC_MEMINFO = '/proc/meminfo'

  # ========================================================================= #
  # === FILE_COLOURS_FOR_THE_HARDWARE_PROJECT
  # ========================================================================= #
  FILE_COLOURS_FOR_THE_HARDWARE_PROJECT =
    "#{PROJECT_YAML_DIRECTORY}colours_for_the_hardware_information_project.yml"

  # ========================================================================= #
  # === HardwareInformation.kb_to_mb(i)
  # ========================================================================= #
  def self.kb_to_mb(i)
    (
      i.gsub(/ kB/,'').to_i / 1024
    ).to_s+ ' MB'
  end

  # ========================================================================= #
  # === HardwareInformation.is_on_windows?
  # ========================================================================= #
  def self.is_on_windows?
    if RUBY_PLATFORM.include?('win') or # if on windows.
       RUBY_PLATFORM.include?('mingw')
      return true
    end
    return false
  end

  # ========================================================================= #
  # === LSCPU
  #
  # The `lscpu` command may fail, which is why it has to be put into
  # a begin/rescue clause.
  # ========================================================================= #
  if is_on_windows?
    LSCPU = nil
  else
    begin
      LSCPU = `lscpu`.split("\n")
    rescue Errno::ENOENT
      puts 'No binary called lscpu could be found.'
    end
  end

  # ========================================================================= #
  # === CPUINFO
  #
  # We must first check whether the target location actually does happen
  # to exist.
  # ========================================================================= #
  _ = CPUINFO_LOCATION
  if File.exist? _
    CPUINFO = File.read(_)
  else
    CPUINFO = nil
  end

  # ========================================================================= #
  # === SPLITTED_CPUINFO
  # ========================================================================= #
  if CPUINFO
    SPLITTED_CPUINFO = CPUINFO.split(N)
    begin
      LSPCI = `lspci`
    rescue # Must rescue here because lspci might not be available.
      LSPCI = nil
    end
  end

  # ========================================================================= #
  # === DATASET_FROM_PROC_MEMINFO
  # ========================================================================= #
  if File.exist? PROC_MEMINFO
    DATASET_FROM_PROC_MEMINFO = File.readlines(PROC_MEMINFO)
    _ = DATASET_FROM_PROC_MEMINFO.select {|entry|
      entry.include? 'MemTotal'
    }.first.strip.split(':').last.strip
    # ======================================================================= #
    # We discard all but the gigs for RAM.
    # ======================================================================= #
    _ = (_.gsub(/ kB/,'').to_i / 1024).to_s+ ' MB'
  else
    DATASET_FROM_PROC_MEMINFO = nil
  end

  # ========================================================================= #
  # === RAM_AVAILABLE
  # ========================================================================= #
  RAM_AVAILABLE = _

  if DATASET_FROM_PROC_MEMINFO
    _ = DATASET_FROM_PROC_MEMINFO.select {|entry|
        entry.include? 'MemFree'
      }.first.strip.split(':').last.strip #.gsub(/ kB/,'').to_i / 1024).to_s+ ' MB'
    # ======================================================================= #
    # === RAM_AVAILABLE_FREE
    # ======================================================================= #
    RAM_AVAILABLE_FREE = kb_to_mb(_)
  end

end