#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
class HardwareInformation

  # ========================================================================= #
  # === set_run_already
  # ========================================================================= #
  def set_run_already(i)
    @run_already = i
  end

  # ========================================================================= #
  # === pad?
  # ========================================================================= #
  def pad?
    @default_padding
  end

  # ========================================================================= #
  # === append_newline
  # ========================================================================= #
  def append_newline
    add N
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    e string?
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @string
  end; alias string string? # === string

  # ========================================================================= #
  # === set_commandline_mode
  # ========================================================================= #
  def set_commandline_mode
    @mode = :commandline
  end

  # ========================================================================= #
  # === set_web_mode
  # ========================================================================= #
  def set_web_mode
    @mode = :web
  end

  # ========================================================================= #
  # === default_ljust
  # ========================================================================= #
  def default_rjust(i)
    if i.is_a? Array # We do not handle Arrays.
      i
    else
      i.to_s.rjust(MAIN_DEFAULT_RJUST)
    end
  end

  # ========================================================================= #
  # === _                                                             (_ tag)
  #
  # This method, as a shortcut, makes use of the method called add(),
  # but it will also append a newline to the given input and support
  # colours (if given as argument).
  # ========================================================================= #
  def _(i, optional_colour = nil)
    if optional_colour and use_colours? # Add colours only if needed.
      if optional_colour.is_a? Symbol
        i = ::Colours.send(optional_colour, i)
      else
        i = optional_colour+i+rev
      end
    end
    add i+N # We try it with mandatory newline for now.
  end

  # ========================================================================= #
  # === add_title
  # ========================================================================= #
  def add_title
    optional_colour = nil
    if use_colours?
      if Object.const_defined? :Colours # User has the colours gem installed.
        optional_colour = :mediumslateblue
      else
        optional_colour = green?
      end
    end
    _ N+'Hardware Information'+N, optional_colour
  end

  # ========================================================================= #
  # === colourize_filesize_of_the_harddisc
  # ========================================================================= #
  def colourize_filesize_of_the_harddisc(i)
    Colours.send(
      @hash_use_these_colours[:filesize_of_the_harddisc],
      i
    )
  end

  # ========================================================================= #
  # === colourize_file_system_type
  # ========================================================================= #
  def colourize_file_system_type(i)
    Colours.send(
      @hash_use_these_colours[:file_system_type],
      i
    )
  end

  # ========================================================================= #
  # === colourize_n_percent_occupied
  # ========================================================================= #
  def colourize_n_percent_occupied(i)
    Colours.send(
      @hash_use_these_colours[:n_percent_occupied],
      i
    )
  end

  # ========================================================================= #
  # === colourize_highlighted_content
  # ========================================================================= #
  def colourize_highlighted_content(i)
    Colours.send(
      @hash_use_these_colours[:highlighted_content],
      i
    )
  end

  # ========================================================================= #
  # === colourize_via_regular_colour
  #
  # Do note that this is presently hardcoded to the "thistle" colour.
  # Unsure whether to allow the user more customization here, so for
  # the time being this remains hardcoded.
  # ========================================================================= #
  def colourize_via_regular_colour(i)
    Colours.send(
      :thistle,
      i
    )
  end

  # ========================================================================= #
  # === append_output_generated_via_lsblk
  #
  # Typical output generated via lsblk may be like this:
  #
  #   NAME   MAJ:MIN RM  SIZE RO TYPE MOUNTPOINT
  #   sda      8:0    0  2.7T  0 disk 
  #   ├─sda1   8:1    0    1M  0 part 
  #   └─sda2   8:2    0  2.7T  0 part /
  #   sr0     11:0    1 1024M  0 rom
  #
  # ========================================================================= #
  def append_output_generated_via_lsblk
    _ = ''.dup
    # ======================================================================== #
    # Next we will display/obtain more information, via df.
    # ======================================================================== #
    extended_information = `df -T`.split(N)
    result = `lsblk`.strip.split(N).map {|line| "    #{line}" }
    result.each {|entry|
      # ===================================================================== #
      # Modify the input.
      # ===================================================================== #
      if entry.include? ' /'
        # =================================================================== #
        # These entries will be colourized. Note that the first part may
        # look like this: "└─sda2"
        #
        #   "    └─sda2   8:2    0  2.7T  0 part /"
        #
        # =================================================================== #
        splitted = entry.split(' ')
        # =================================================================== #
        # The root_entry may look like this:
        #
        #   /dev/sda2      ext4     2883218608 529813416 2206875608  20% /
        #
        # =================================================================== #
        selection = extended_information.select {|line|
          last = line.split(' ').last
          last == '/' or last.include?('USB')
        }
        if selection
          selection = selection.select {|inner_line|
            inner_line.end_with? splitted[-1]
          }
          if selection.is_a?(Array) and selection.first
            selection = selection.first.split(' ')
          end
        end
        splitted[-1] = colourize_highlighted_content(splitted[-1].ljust(12)) # ← The last part.
        splitted[0] = colourize_highlighted_content("    #{splitted[0]}")    # ← The first part.
        splitted[1] = colourize_via_regular_colour(splitted[1].strip.ljust(4).rjust(6))
        splitted[2] = colourize_via_regular_colour(" #{splitted[2].strip.rjust(2)}")
        splitted[3] = colourize_filesize_of_the_harddisc(splitted[3].strip.rjust(5).ljust(6)) # ← The size of the harddisc.
        splitted[4] = colourize_via_regular_colour(splitted[4].strip.ljust(2).rjust(3))
        splitted[5] = colourize_via_regular_colour(splitted[5].strip)
        file_system_type   = selection[1]
        n_percent_occupied = selection[5]
        # =================================================================== #
        # /dev/sda2   ext4    2883218608 529753916 2206935108  20% /"]
        # =================================================================== #
        splitted = splitted.join(' ')
        unless entry.include? '/boot/' # ← Don't show information for /boot/ entries.
          splitted << colourize_file_system_type(" (#{file_system_type})")
          splitted << colourize_n_percent_occupied(" (#{n_percent_occupied} used)")
        end
        entry = splitted
      else # Else we only do a slightly simpler colourization-part.
        splitted = entry.split(' ')
        splitted[0] = colourize_via_regular_colour("    #{splitted[0].ljust(6)}")      # ← The first part.
        splitted[1] = colourize_via_regular_colour(splitted[1].strip.ljust(4).rjust(6))
        splitted[2] = colourize_via_regular_colour(" #{splitted[2].strip.rjust(2)}")
        splitted[3] = colourize_filesize_of_the_harddisc(splitted[3].strip.rjust(5).ljust(6)) # ← The size of the harddisc.
        splitted[4] = colourize_via_regular_colour(splitted[4].strip.ljust(2).rjust(3))
        splitted[5] = colourize_via_regular_colour(splitted[5].strip)
        if splitted[6] and (splitted[6] == '[SWAP]')
          splitted[6] = royalblue(splitted[6])
        end
        entry = splitted.join(' ')
        entry = colourize_via_regular_colour(entry)
      end
      _ << "#{entry}\n" # Now we can append this.
    }
    # ======================================================================= #
    # Add two newlines too, for slightly more readable output.
    # ======================================================================= #
    _ << "#{N}#{N}"
    add _
  end

  # ========================================================================= #
  # === default_ljust
  # ========================================================================= #
  def default_ljust(i)
    i.to_s.ljust(15)
  end

  # ========================================================================= #
  # === add_audio_information                                     (audio tag)
  # ========================================================================= #
  def add_audio_information
    audio_device = `aplay -l`.scan(/card.+$/).first
    make_line(pad?+'Audio Device', N+(pad? * 2)+audio_device.to_s)
  end

  # ========================================================================= #
  # === find_out_the_amount_of_processors
  # ========================================================================= #
  def find_out_the_amount_of_processors
    data = CPUINFO.scan(/cpu cores\t: (\d+)\n/).flatten.map(&:to_i).sum.to_i
    make_line('  Processors', data) { :slateblue }
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  # ========================================================================= #
  def add(i)
    @string << i
  end

end