#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This page should be kept in german.
# =========================================================================== #
# http://localhost/DATA/PROGRAMMING_LANGUAGES/RUBY/src/hardware_information/lib/hardware_information/www/my_hardware.cgi
# =========================================================================== #
# rffhware
# =========================================================================== #
require 'hardware_information/misc/purchased_hardware/purchased_hardware.rb'

# =========================================================================== #
# Note that the main hardware-related dataset is stored in this file:
#
#   bl /home/x/data/personal/yaml/my_hardware/my_hardware.yml
#
# =========================================================================== #
german {
autoextend
title HardwareInformation::PurchasedHardware.title?
favicon 'PC/HARDWARE/HWARE_FAVICON.png'
css_style '
pre {
  font-size: 1.3em;
  color:green;
}
body {
  padding-left:  8px;
  padding-right: 8px;  
  font-weight:   normal;
  color:         white;
}
div {
  font-weight:   normal;
}
h1,h2,h3,h4,h5,h6 {
  margin-top: 2px;
  padding:    2px;
}

a            { color: steelblue; text-decoration: none; }
a:link       { color: steelblue; }
a:visited    { color: steelblue; }
a:hover      { color: mediumspringgreen; text-decoration: none; }
a:active     { color: steelblue ; }
a            { color: steelblue; }
i            { font-size: bigger; font-weight:500}
img          { margin-right: 2px;}'
body_css_class HardwareInformation::PurchasedHardware.body_css_class
body_css_style HardwareInformation::PurchasedHardware.body_css_style

font_size '1em'

EURO_IMAGE      = sg(:euro, 'mar1px mars5px').dup
MARGIN_RIGHT    = 'marr8px'
S_CHAOS         = sg :square_chaos,         MARGIN_RIGHT
S_DARKGOLDENROD = sg :square_darkgoldenrod, MARGIN_RIGHT
GREEN_SQUARE    = sg(:green_square, "bblack1 #{MARGIN_RIGHT}")

# =========================================================================== #
# === external_link
# =========================================================================== #
def external_link(
    remote,
    name,
    optional_use_this_id = nil
  )
  abr(
    remote,
    content:   S_CHAOS+name,
    css_class: 's1em',
    id: optional_use_this_id
  )
end

# =========================================================================== #
# === show_local_sitemap
# =========================================================================== #
def show_local_sitemap
  h2 'Sitemap','yel martb1px'
  p('marr1em FS1em mart0_1em'){
    sitemap_link '#Laptop1_Fujitsu_Siemens_Computers',
                 'Laptop FSC Amilo Pro V2030'
    sitemap_link '#USB_Netgear_108_WG111T',
                 'USB Netgear 108 WG111T Sticks '
    sitemap_link '#Maxtor_OneTouch_III_750',
                 'Maxtor External USB HDD 750 GIG'
    sitemap_link '#Imation_clip_Flash_Drive',
                 'Imitation 4 GIG USB Stick'
    sitemap_link '#Buffalo_Wireless_G_125',
                 'Buffalo Wireless Router'
    sitemap_link '#computer3',
                 'Rechner Computer3' # Mit K8M890
    sitemap_link '#Rechner_Miraculix_mit_xxxxx',
                 'Rechner Miraculix'
    sitemap_link '#Benq_G2000WA_LCD_Monitor',
                 'Benq LCD Monitor'
    sitemap_link '#Pinnacle_PCTV_Hybrid_Pro',
                 'Pinnacle PCTV Hybrid Pro Stick 330E'
    sitemap_link '#LCD_TV_19LG3000',
                 'LCD TV 19LG3000'
    sitemap_link '#FREECOM_EXTERNAL_HARD_DRIVE_3TB',
                 'FREECOM_EXTERNAL_HARD_DRIVE_3TB'
  }
end

# =========================================================================== #
# === discarded_on_this_day
# =========================================================================== #
def discarded_on_this_day(i = nil)
  if i
    s2 sg(:rip,'marr8px').dup+
       '<span class="cyan">Entsorgt am: </span>','padl2_5em'
    s2 i.to_s, 'BO green'
  end
end

# =========================================================================== #
# === sitemap_link
# =========================================================================== #
def sitemap_link(intralink, desc)
  unless intralink.start_with? '#'
    intralink[0, 0] = '#'
  end
  abr(
    intralink,
    {
      content:      S_DARKGOLDENROD+desc,
      css_class:    'slateblue'
    }
  )
end

# =========================================================================== #
# === show_interesting_hardware_that_could_be_purchased
# =========================================================================== #
def show_interesting_hardware_that_could_be_purchased
  # ========================================================================= #
  # === Hardware die ich kaufen könnte
  # ========================================================================= #
  h3 'Interessante Hardware die ich kaufen könnte:'
  div {
    # Tablets gibt es bereits ab €282,92 Euro.
    #
    #
    # === https://www.notebookcheck.net/Trekstor-Primetab-T13B-N4200-HD505-Convertible-Review.324136.0.html
    # === https://geizhals.at/trekstor-primetab-t13b-64gb-grau-99403-a1827199.html
    # https://geizhals.at/trekstor-primetab-s11b-64gb-grau-99003-a2169278.html?hloc=at
    # ^^^  Gewicht 780g (mit Dock: 1.06kg)
    e 'TrekStor Primetab T13B 64GB grey (99403): ab 368 euro'
    e 'Stand: ^^^ 13.11.2018'
    br
    e 'Realtek USB: RTL8188CUS chip'
    e 'The kernel driver is called: rtl8192cu'
    e 'USB: 0846:9041 NetGear, Inc. WNA1000M 802.11bgn [Realtek RTL8188CUS]'
    br
    a 'https://geizhals.at/lenovo-ideapad-miix-320-10icr-80xf002hge-a1616381.html?hloc=at'
    a 'https://geizhals.at/trekstor-primetab-t13b-64gb-grau-99403-a1827199.html'
  }
end

# =========================================================================== #
# === show_external_links
# =========================================================================== #
def show_external_links
  # ========================================================================= #
  # LINKS ZU HARDWARE
  # ========================================================================= #
  div(HardwareInformation::PurchasedHardware::MAIN_DIV,'',
      'margin-left: 1em;
       margin-right: 1em;
       border:1px ridge lightblue'){
    h2 sg(:big_star,'marr5px','drag_the_big_star')+
      'Links',
      'marl0_5em mart10px mar2px pad4px darkkhaki'
    div('mars2em s1em pad10px mar8px','','border:1px dashed yellow'){
      external_link 'https://www.motherboards.org/',
                    'Motherboards'
      external_link 'https://kmuto.jp/debian/hcl/index.cgi',
                    'Debian Hardware Page'      
      external_link 'https://geizhals.at/?m=1',
                    'Geizhals'
      external_link 'https://www.hardwareschrott.de/',
                    'Hardware Schrott','Hardware_Schrott'
      external_link 'https://www.rudiratz.de/index.php?m=2',
                    'Computer Lexikon'
      external_link 'https://www.alleseineuro.de/article_info.php/articles_id/1',
                    'Richtiges Löten'
      external_link 'https://www.tomshardware.com/',
                    'Toms Hardware Tips'
      external_link 'https://www.pcmasters.de/',
                    'Hardware News'
      external_link 'https://de.wikipedia.org/wiki/Liste_der_BIOS-Signalt%C3%B6ne',
                    'Biep Fehlercodes (Wikipedia)'
      external_link 'https://www.website-go.com/artikel/beepcodes.php',
                    'Gute Biep Fehlercodes mit Hilfestellung'
      external_link 'https://www.website-go.com/artikel/beepcodes.php',
                    'Deutschprachige Biep Fehlercodes'
      external_link 'https://www.alltests.de/index.php?sel=php/news/shortnews.php',
                    'Gute Hardware Tips'
      external_link 'https://www.digiwelt.at/officewelt-at/cf/frameset.cfm?bid=D3D9446802A44259755D38E6D163E820&fuseaction=Artikel.detail&artikel_id=1468',
                    'Hardware bei InterSpar'
      external_link :local_hardware_assembly,
                    'Lokaler Link zu Hardware Assembly'
      external_link :wlan,
                    'Lokaler Link zu WLAN'
      external_link :local_ram,
                   'Lokaler Link zu RAM'
      external_link :local_scanner,
                   'Lokaler Link zu SCANNERS'
      external_link :local_sitemap,
                   'Lokaler Link zur SITEMAP'
    }
  }
end

# =========================================================================== #
# === display_the_total_costs
#
# This method will show the sum of the total purchase cost; the module
# PurchasedHardware will keep track of this, so here we only display
# that value.
# =========================================================================== #
def display_the_total_costs(
    i = HardwareInformation::PurchasedHardware.total_cost?
  )
  # ========================================================================= #
  # === REGISTRIERTE GESAMTKOSTEN
  # ========================================================================= #
  div('mart1em s1em') {
    e sg(:dot102, 'marr8px')+
      '<b class="marl0_3em bigger"> Registrierte Gesamtkosten:</b>'
    br
    s2 i.to_f.round(3).to_s,'yel marl2em BOLD bigger'
    s2 EURO_IMAGE+
       ' <b>Euro</b> <span class="smaller">(seit 27.05.2004)</span>'
    br
    br
  }
end

doc(HardwareInformation::PurchasedHardware::MAIN_DIV,'','min-height:32em') {
  img 'PC/HARDWARE/HARDWARE.png','FLR BG_White',
      'drag_hardware',
      'margin-right:2%; top: 3em; opacity: 0.4;'
  # ========================================================================= #
  # Show a little heart-image next.
  # ========================================================================= #
  h2 sg(:herz, 'marr8px')+
     'Meine Hardware - ausgelesen aus der Datei <br><br><b>'+
      HardwareInformation::PurchasedHardware.file_computer_hardware?+'</b>',
     'BO s0_2em BLOCK mart1px marb1em',
     '','padding-top:0.05em'
  ehtml_comment('Start of MyHardware')
  # ========================================================================= #
  # Query our main hash next.
  # ========================================================================= #
  HardwareInformation::PurchasedHardware.my_hardware.each_pair {|name_of_the_hardware_in_question, hash|
    _ = hash
    s2 GREEN_SQUARE+
      name_of_the_hardware_in_question
    # ======================================================================= #
    # Get the id to store onto the coming <div> tag.
    # ======================================================================= #
    the_id_to_store = Cyberweb.remove_html(
      name_of_the_hardware_in_question.tr(' ', '_')
    ).to_s.strip.tr('-','_')
    # ======================================================================= #
    # If we find a newline then we ignore all up to that newline.
    # ======================================================================= #
    if the_id_to_store.include? "\n"
      the_id_to_store = the_id_to_store[0, the_id_to_store.index("\n")]
    end
    if the_id_to_store.include? '"'
      the_id_to_store.delete! '"'
    end
    if the_id_to_store.count('_') > 3 # Only keep the first 3 '_'
      the_id_to_store = the_id_to_store.split('_')[0..3].join('_')
    end
    if the_id_to_store.end_with? '_'
      the_id_to_store.chop!
    end
    # ======================================================================= #
    # The respective <div> for the Hardware in question comes next.
    # ======================================================================= #
    div('mart1em mars1em pad4px pad1em', id: the_id_to_store) {
      # Next a Hash will be used:
      # ===================================================================== #
      # === description
      # ===================================================================== #
      if _.has_key? 'description'
        e _['description']
      end
      # ===================================================================== #
      # === image
      # ===================================================================== #
      if _.has_key? 'image'
        this_image = _['image']
        e sg(
            this_image[0].to_s,
            this_image[1].to_s,
            this_image[2].to_s,
            this_image[3].to_s
          )
      # ===================================================================== #
      # === draggable_image
      # ===================================================================== #
      elsif _.has_key? 'draggable_image'
        this_image = _['draggable_image']
        dimg(this_image[0].to_s, this_image[1].to_s)
        br
      end
      # ===================================================================== #
      # === bought_at
      #
      # When did we buy a specific hardware - the date of purchase. Will
      # be displayed right here.
      # ======================================================================= #
      if _.has_key?('bought_at') and !_['bought_at'].empty?
        bought_at_that_date_specifically = _['bought_at'].to_s.dup
        einkaufswagen = sg(:einkaufswagen, 'marl1em wid25px').dup
        # =================================================================== #
        # Next show when this hardware was purchased.
        # =================================================================== #
        s2 'Gekauft am:',
           'padl2_5em marl1em'
        s2 bought_at_that_date_specifically+
          einkaufswagen,
          'BOLD '+
          HardwareInformation::PurchasedHardware::COLOUR_PURCHASED_AT_THIS_DATE
        # =================================================================== #
        # And register it onto the main module that handles this.
        # =================================================================== #
        HardwareInformation::PurchasedHardware.add_bought_at(
          bought_at_that_date_specifically
        )
      end
      # ===================================================================== #
      # === cost
      # ===================================================================== #
      if _.has_key? 'cost'
        cost = _['cost'].to_s.to_f
        case cost
        when 0
          e '<b class="BO yel larger">Kosten unbekannt</b>'
        else
          HardwareInformation::PurchasedHardware.add_to_the_total_cost(cost)
          e 'um <b class="BO larger yellow">'+
            _['cost'].to_s+'</b>'+
            EURO_IMAGE.to_s
            "\n"' Euro</b> erworben'
        end
      end
      # ===================================================================== #
      # === died_at
      # ===================================================================== #
      if _.has_key? 'died_at'
        discarded_on_this_day(_['died_at'])
      end
      # ===================================================================== #
      # === link
      # ===================================================================== #
      if _.has_key? 'link'
        a(
          _['link'][0].to_s,
          content: _['link'][0].to_s,
          css_class: _['link'][1].to_s
        )
      # ===================================================================== #
      # === links
      # ===================================================================== #
      elsif _.has_key? 'links'
        _['links'].each {|entry, inner_array|
          a(
            inner_array[0].to_s,
            content: inner_array[0].to_s,
            css_class: inner_array[1].to_s
          )
        }
      end
      # ===================================================================== #
      # === images
      #
      # For reference, an entry matching the following clause may look
      # like this:
      #
      #   - PC/HARDWARE/TV/XORO_TV_HTC1546_OUTLETS.jpg
      #   - mar0_3em
      #   - drag_xoro_tv_rückseite
      #
      # ===================================================================== #
      if _.has_key? 'images'
        _['images'].each {|entry_array, inner_array|
          img(
            inner_array[0].to_s,
            content:   inner_array[0].to_s,
            css_class: inner_array[1].to_s,
            the_id:    inner_array[2].to_s,
            css_style: inner_array[3].to_s
          )
        }
      end
    } # this ends the <p> tag.
    spacer(:middle) # Added this spacer as of August 2014.
    hr
  }
  # ========================================================================= #
  # === Display the final total costs next, close to the end of the web-app
  # ========================================================================= #
  display_the_total_costs
  show_external_links
  # ========================================================================= #
  # bl $LINUX/YAML/Computer_hardware.yml
  # ========================================================================= #
  show_interesting_hardware_that_could_be_purchased
  # ========================================================================= #
  # Sitemap Tag. Hier nur Sachen sammeln die halbwegs wichtig sind!
  # Sonst läppert sich das wie verrückt zusammen.
  # ========================================================================= #
  div('wid30 posab pad1em padt0_5em BG_BLACK White','',
      'right:0.5em; top:1em; border:1px dotted slateblue;'){
    show_local_sitemap
  }
  # sandisk USB fehlt noch hmmm.
  #   Siehe hier: https://www.amazon.de/SanDisk-Cruzer-32GB-USB-Stick-Schwarz/dp/B00812F7O8
  # ========================================================================= #
  # Eventuell mini-pc kaufen:
  #
  #   https://liliputing.com/2020/07/asus-pn50-is-a-mini-pc-with-up-to-an-amd-ryzen-7-4800u-processor.html
  #   https://geizhals.at/asus-mini-pc-pn50-v26823.html
  #   https://geizhals.at/asus-mini-pc-pn50-bbr343md-90mr00e1-m00150-a2314947.html
  #   ^^^ 331 Euro am 12.12.2020
  #   ^^^ 328 Euro am 22.01.2021
  #   ^^^ 316 Euro am 31.03.2021
  # ========================================================================= #
  # Oder:
  #
  # https://geizhals.at/shuttle-xpc-slim-pos-ds100-pec-ds10u001-a2221818.html?hloc=at
  # 4 GIG RAM, 299 Euro.
  # ========================================================================= #
}}